local L = AceLibrary("AceLocale-2.2"):new("GridStatusPvP")
local RL = AceLibrary("Roster-2.1")

local PlayerFaction = UnitFactionGroup("player")

L:RegisterTranslations("enUS", function() return {
	["PvP"] = true,
	["<PvP>"] = true,
	["<PvP-FFA>"] = true,
	["PvP alert"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["PvP"] = "PvP",
	["<PvP>"] = "<PvP>",
	["<PvP-FFA>"] = "<PvP-FFA>",
	["PvP alert"] = "PvP Alarm",
} end)

L:RegisterTranslations("zhCN", function() return {
	["PvP"] = "PvP",
	["<PvP>"] = "<PvP>",
	["<PvP-FFA>"] = "<PvP-ʰȡ>",
	["PvP alert"] = "PvP ",
} end)

L:RegisterTranslations("zhTW", function() return {
	["PvP"] = "PvP",
	["<PvP>"] = "<PvP>",
	["<PvP-FFA>"] = "<PvP-ʰȡ>",
	["PvP alert"] = "PvP ",
} end)

GridStatusPvP = GridStatus:NewModule("GridStatusPvP")
GridStatusPvP.menuName = L["PvP"]

GridStatusPvP.defaultDB = {
	debug = false,
	alert_pvp = {
		text =  L["<PvP>"],
		enable = false,
		color = { r = 0.0, g = 1.0, b = 1.0, a = 0.75 },
		priority = 20,
		range = false,
	},
}

GridStatusPvP.options = false

function GridStatusPvP:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("alert_pvp", L["PvP alert"], nil, true)
end

function GridStatusPvP:OnEnable()
	self:RegisterEvent("UNIT_FACTION")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("Grid_UnitJoined")
end

function GridStatusPvP:ZONE_CHANGED_NEW_AREA()
	-- if we are in bg/arena: clear all pvp statuses
	-- if we are NOT in bg/arena: check all units for pvp
	local inInstance, instanceType = IsInInstance()
	if inInstance and (instanceType == "pvp" or instanceType == "arena") then
		self:ClearAllUnits()
	else
		self:UpdateAllUnits()
	end
end

function GridStatusPvP:UNIT_FACTION(unitid)
	if string.find(unitid, "pet") then return end
	-- if we are in bg/arena: return -> do nothing
	local inInstance, instanceType = IsInInstance()
	if inInstance and (instanceType == "pvp" or instanceType == "arena") then
		return
	end
	if UnitIsPVP(unitid) or UnitIsPVPFreeForAll(unitid) then
		self:UnitHasPvP(unitid)
	else
		self.core:SendStatusLost(UnitName(unitid), "alert_pvp")
	end
end

function GridStatusPvP:Grid_UnitJoined(name, unitid)
	-- if we are in bg/arena: do not update status if someone join group
	local inInstance, instanceType = IsInInstance()
	if inInstance and (instanceType == "pvp" or instanceType == "arena") then
		return
	end
	self:UNIT_FACTION(unitid)
end

function GridStatusPvP:ClearAllUnits()
	for u in RL:IterateRoster() do
		self.core:SendStatusLost(UnitName(u.unitid), "alert_pvp")
	end
end

function GridStatusPvP:UpdateAllUnits()
	for u in RL:IterateRoster() do
		if UnitIsPVP(u.unitid) or UnitIsPVPFreeForAll(u.unitid) then
			self:UnitHasPvP(u.unitid)
		end
	end
end

function GridStatusPvP:UnitHasPvP(unitid)
	local settings = self.db.profile.alert_pvp
	local text = L["<PvP>"]
	local icon = "Interface\\AddOns\\GridStatusPvP\\icons\\PvP-Horde.tga"
	if PlayerFaction == "Alliance" then
		icon = "Interface\\AddOns\\GridStatusPvP\\icons\\PvP-Alliance.tga"
	end
	if UnitIsPVPFreeForAll(unitid) then
		text = L["<PvP-FFA>"]
		icon = "Interface\\AddOns\\GridStatusPvP\\icons\\PvP-FFA.tga"
	end
	self.core:SendStatusGained(UnitName(unitid), "alert_pvp",
		settings.priority,
		(settings.range and 40),
		settings.color,
		text,
		nil,
		nil,
		icon)
end